export var AuthErrorCode;
(function (AuthErrorCode) {
    AuthErrorCode["USER_NOT_FOUND"] = "USER_NOT_FOUND";
    AuthErrorCode["USER_DISABLED"] = "USER_DISABLED";
    AuthErrorCode["INVALID_CREDENTIAL"] = "INVALID_CREDENTIAL";
    AuthErrorCode["TOKEN_EXPIRED"] = "TOKEN_EXPIRED";
    AuthErrorCode["TOKEN_REVOKED"] = "TOKEN_REVOKED";
    AuthErrorCode["INVALID_ARGUMENT"] = "INVALID_ARGUMENT";
    AuthErrorCode["INTERNAL_ERROR"] = "INTERNAL_ERROR";
    AuthErrorCode["NO_KID_IN_HEADER"] = "NO_KID_IN_HEADER";
    AuthErrorCode["INVALID_SIGNATURE"] = "INVALID_SIGNATURE";
    AuthErrorCode["NO_MATCHING_KID"] = "NO_MATCHING_KID";
    AuthErrorCode["MISMATCHING_TENANT_ID"] = "MISMATCHING_TENANT_ID";
})(AuthErrorCode || (AuthErrorCode = {}));
const AuthErrorMessages = {
    [AuthErrorCode.USER_NOT_FOUND]: 'User not found',
    [AuthErrorCode.INVALID_CREDENTIAL]: 'Invalid credentials',
    [AuthErrorCode.TOKEN_EXPIRED]: 'Token expired',
    [AuthErrorCode.USER_DISABLED]: 'User disabled',
    [AuthErrorCode.TOKEN_REVOKED]: 'Token revoked',
    [AuthErrorCode.INVALID_ARGUMENT]: 'Invalid argument',
    [AuthErrorCode.INTERNAL_ERROR]: 'Internal error',
    [AuthErrorCode.NO_KID_IN_HEADER]: 'No kid in jwt header',
    [AuthErrorCode.INVALID_SIGNATURE]: 'Invalid token signature.',
    [AuthErrorCode.NO_MATCHING_KID]: 'Kid is not matching any certificate',
    [AuthErrorCode.MISMATCHING_TENANT_ID]: 'Provided tenant ID does not match firebase tenant ID from the token'
};
function getErrorMessage(code, customMessage) {
    if (!customMessage) {
        return AuthErrorMessages[code];
    }
    return `${AuthErrorMessages[code]}: ${customMessage}`;
}
function mergeStackTraceAndCause(target, original) {
    const originalError = original;
    const originalErrorStack = typeof originalError?.stack === 'string' ? originalError.stack : '';
    const originalCause = typeof originalError?.cause === 'string' ? originalError.cause : '';
    target.stack = (target?.stack ?? '') + originalErrorStack;
    target.cause = (target?.cause ?? '') + originalCause;
}
export class AuthError extends Error {
    static fromError(error, code, customMessage) {
        const authError = new AuthError(code, customMessage);
        mergeStackTraceAndCause(authError, error);
        return authError;
    }
    constructor(code, customMessage) {
        super(getErrorMessage(code, customMessage));
        this.code = code;
        Object.setPrototypeOf(this, AuthError.prototype);
    }
    toJSON() {
        return {
            code: this.code,
            message: this.message
        };
    }
}
export var InvalidTokenReason;
(function (InvalidTokenReason) {
    InvalidTokenReason["MISSING_CREDENTIALS"] = "MISSING_CREDENTIALS";
    InvalidTokenReason["MISSING_REFRESH_TOKEN"] = "MISSING_REFRESH_TOKEN";
    InvalidTokenReason["MALFORMED_CREDENTIALS"] = "MALFORMED_CREDENTIALS";
    InvalidTokenReason["INVALID_SIGNATURE"] = "INVALID_SIGNATURE";
    InvalidTokenReason["INVALID_CREDENTIALS"] = "INVALID_CREDENTIALS";
    InvalidTokenReason["INVALID_KID"] = "INVALID_KID";
})(InvalidTokenReason || (InvalidTokenReason = {}));
const InvalidTokenMessages = {
    [InvalidTokenReason.MISSING_CREDENTIALS]: 'Missing credentials',
    [InvalidTokenReason.MALFORMED_CREDENTIALS]: 'Credentials are incorrectly formatted',
    [InvalidTokenReason.INVALID_SIGNATURE]: 'Credentials have invalid signature',
    [InvalidTokenReason.MISSING_REFRESH_TOKEN]: 'Refresh token is missing',
    [InvalidTokenReason.INVALID_CREDENTIALS]: 'Invalid credentials',
    [InvalidTokenReason.INVALID_KID]: 'Token has kid claim that cannot be matched with any known Google certificate. This usually indicates that Google certificates have expired and user has to reauthenticate.'
};
export class InvalidTokenError extends Error {
    static fromError(error, reason) {
        const invalidTokenError = new InvalidTokenError(reason);
        mergeStackTraceAndCause(invalidTokenError, error);
        return invalidTokenError;
    }
    constructor(reason) {
        super(`${reason}: ${InvalidTokenMessages[reason]}`);
        this.reason = reason;
        this.isInvalidTokenError = true;
        Object.setPrototypeOf(this, InvalidTokenError.prototype);
    }
}
export function isInvalidTokenError(error) {
    return error?.isInvalidTokenError ?? false;
}
