import { base64url } from 'jose';
import { fetchText } from '../utils.js';
import { sign, signBlob } from './sign.js';
import { ALGORITHM_RS256 } from './verify.js';
export function createEmulatorToken(payload) {
    const header = {
        alg: 'none',
        typ: 'JWT'
    };
    return `${base64url.encode(JSON.stringify(header))}.${base64url.encode(JSON.stringify(payload))}.${base64url.encode('')}`;
}
export class EmulatorSigner {
    constructor(tenantId) {
        this.tenantId = tenantId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        return createEmulatorToken(payload);
    }
    getAccountId() {
        return Promise.resolve('firebase-auth-emulator@example.com');
    }
}
export class ServiceAccountSigner {
    constructor(credential, tenantId) {
        this.credential = credential;
        this.tenantId = tenantId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        return sign({ payload, privateKey: this.credential.privateKey });
    }
    getAccountId() {
        return Promise.resolve(this.credential.clientEmail);
    }
}
export class IAMSigner {
    constructor(credential, tenantId, serviceAccountId) {
        this.algorithm = ALGORITHM_RS256;
        this.credential = credential;
        this.tenantId = tenantId;
        this.serviceAccountId = serviceAccountId;
    }
    async sign(payload) {
        if (this.tenantId) {
            payload.tenant_id = this.tenantId;
        }
        const serviceAccount = await this.getAccountId();
        const accessToken = await this.credential.getAccessToken();
        return signBlob({
            accessToken: accessToken.accessToken,
            serviceAccountId: serviceAccount,
            payload
        });
    }
    async getAccountId() {
        if (this.serviceAccountId) {
            return this.serviceAccountId;
        }
        const token = await this.credential.getAccessToken();
        const url = 'http://metadata/computeMetadata/v1/instance/service-accounts/default/email';
        const request = {
            method: 'GET',
            headers: {
                'Metadata-Flavor': 'Google',
                Authorization: `Bearer ${token.accessToken}`
            }
        };
        return (this.serviceAccountId = await fetchText(url, request));
    }
}
