import { RotatingCredential } from '../../../auth/rotating-credential.js';
export class MultipleCookieBuilder {
    constructor(cookieName, signatureKeys) {
        this.cookieName = cookieName;
        this.credential = new RotatingCredential(signatureKeys);
    }
    async buildCookies(tokens) {
        const signature = await this.credential.createSignature(tokens);
        if (!tokens.customToken) {
            return [
                {
                    name: `${this.cookieName}.id`,
                    value: tokens.idToken
                },
                {
                    name: `${this.cookieName}.refresh`,
                    value: tokens.refreshToken
                },
                {
                    name: `${this.cookieName}.sig`,
                    value: signature
                }
            ];
        }
        return [
            {
                name: `${this.cookieName}.id`,
                value: tokens.idToken
            },
            {
                name: `${this.cookieName}.refresh`,
                value: tokens.refreshToken
            },
            {
                name: `${this.cookieName}.custom`,
                value: tokens.customToken
            },
            {
                name: `${this.cookieName}.sig`,
                value: signature
            }
        ];
    }
}
