import { errors } from 'jose';
import { InvalidTokenError, InvalidTokenReason } from '../../../auth/error.js';
import { RotatingCredential } from '../../../auth/rotating-credential.js';
export class MultipleCookiesParser {
    constructor(cookies, cookieName, signatureKeys) {
        this.cookies = cookies;
        this.cookieName = cookieName;
        this.signatureKeys = signatureKeys;
    }
    async parseCookies() {
        const idTokenCookie = this.cookies.get(`${this.cookieName}.id`);
        const refreshTokenCookie = this.cookies.get(`${this.cookieName}.refresh`);
        const customTokenCookie = this.cookies.get(`${this.cookieName}.custom`);
        const signatureCookie = this.cookies.get(`${this.cookieName}.sig`);
        if (![idTokenCookie, refreshTokenCookie, signatureCookie].every(Boolean)) {
            throw new InvalidTokenError(InvalidTokenReason.MISSING_CREDENTIALS);
        }
        const signature = signatureCookie;
        const customTokens = {
            idToken: idTokenCookie,
            refreshToken: refreshTokenCookie,
            customToken: customTokenCookie
        };
        const credential = new RotatingCredential(this.signatureKeys);
        try {
            await credential.verifySignature(customTokens, signature);
            return customTokens;
        }
        catch (e) {
            if (e instanceof errors.JWSSignatureVerificationFailed) {
                throw new InvalidTokenError(InvalidTokenReason.INVALID_SIGNATURE);
            }
            throw e;
        }
    }
}
