import { FlattenedSign, SignJWT, errors, jwtVerify } from 'jose';
import { toUint8Array } from '../utils.js';
export async function createCustomSignature(tokens, key) {
    const jws = await new FlattenedSign(toUint8Array(tokens.customToken
        ? `${tokens.idToken}.${tokens.refreshToken}.${tokens.customToken}`
        : `${tokens.idToken}.${tokens.refreshToken}`))
        .setProtectedHeader({ alg: 'HS256' })
        .sign(toUint8Array(key));
    return jws.signature;
}
export async function verifyCustomSignature(tokens, signature, key) {
    if ((await createCustomSignature(tokens, key)) !== signature) {
        throw new errors.JWSSignatureVerificationFailed('');
    }
}
export async function createCustomJWT(payload, secret) {
    const jwt = new SignJWT(payload);
    jwt.setProtectedHeader({ alg: 'HS256', typ: 'JWT' });
    return jwt.sign(toUint8Array(secret));
}
export async function verifyCustomJWT(customJWT, secret) {
    return jwtVerify(customJWT, toUint8Array(secret));
}
