import { errors } from 'jose';
import { createCustomJWT, createCustomSignature, verifyCustomJWT, verifyCustomSignature } from './custom-token/index.js';
export class RotatingCredential {
    keys;
    constructor(keys) {
        this.keys = keys;
    }
    async sign(payload) {
        return createCustomJWT(payload, this.keys[0]);
    }
    async createSignature(tokens) {
        return createCustomSignature(tokens, this.keys[0]);
    }
    async verify(customJWT) {
        for (const key of this.keys) {
            try {
                const result = await verifyCustomJWT(customJWT, key);
                return result.payload;
            }
            catch (e) {
                if (e instanceof errors.JWSSignatureVerificationFailed ||
                    e instanceof errors.JWSInvalid) {
                    continue;
                }
                throw e;
            }
        }
        throw new errors.JWSSignatureVerificationFailed('Custom JWT could not be verified against any of the provided keys');
    }
    async verifySignature(tokens, signature) {
        for (const key of this.keys) {
            try {
                return await verifyCustomSignature(tokens, signature, key);
            }
            catch (e) {
                if (e instanceof errors.JWSSignatureVerificationFailed ||
                    e instanceof errors.JWSInvalid) {
                    continue;
                }
                throw e;
            }
        }
        throw new errors.JWSSignatureVerificationFailed('Custom tokens signature could not be verified against any of the provided keys');
    }
}
