import { getFirebaseAuth } from '../auth/index.js';
import { AuthCookies } from './cookies/AuthCookies.js';
import { ObjectCookiesProvider } from './cookies/parser/ObjectCookiesProvider.js';
import { getCookiesTokens } from './tokens.js';
export async function refreshApiResponseCookies(request, response, options) {
    const tokens = await refreshApiCookies(request.cookies, request.headers, options);
    await appendAuthCookiesApi(request.cookies, response, tokens, options);
    return response;
}
export async function appendAuthCookiesApi(cookies, response, tokens, options) {
    const authCookies = new AuthCookies(new ObjectCookiesProvider(cookies), options);
    await authCookies.setAuthNextApiResponseHeaders(tokens, response);
}
export async function refreshApiCookies(cookies, headers, options) {
    const referer = headers['referer'] ?? '';
    const tokens = await getCookiesTokens(cookies, options);
    const { handleTokenRefresh } = getFirebaseAuth({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const tokenRefreshResult = await handleTokenRefresh(tokens.refreshToken, {
        referer,
        enableCustomToken: options.enableCustomToken
    });
    return {
        customToken: tokenRefreshResult.customToken,
        idToken: tokenRefreshResult.idToken,
        refreshToken: tokenRefreshResult.refreshToken,
        decodedIdToken: tokenRefreshResult.decodedIdToken
    };
}
