import { CookieParserFactory } from '../parser/CookieParserFactory.js';
import { CombinedCookieRemover } from './CombinedCookieRemover';
import { MultipleCookieRemover } from './MultipleCookieRemover';
import { SingleCookieRemover } from './SingleCookieRemover';
export class CookieRemoverFactory {
    static fromRequestCookies(cookies, provider, cookieName) {
        const singleCookie = provider.get(cookieName);
        const hasEnabledMultipleCookies = CookieParserFactory.hasMultipleCookies(provider, cookieName);
        const hasEnabledLegacyMultipleCookies = CookieParserFactory.hasLegacyMultipleCookies(provider, cookieName);
        if (singleCookie &&
            (hasEnabledMultipleCookies || hasEnabledLegacyMultipleCookies)) {
            return new CombinedCookieRemover(new MultipleCookieRemover(cookieName, cookies), new SingleCookieRemover(cookieName, cookies));
        }
        if (hasEnabledMultipleCookies) {
            return new MultipleCookieRemover(cookieName, cookies);
        }
        return new SingleCookieRemover(cookieName, cookies);
    }
}
