import { NextResponse } from 'next/server';
import { appendAuthCookies, verifyNextCookies } from './cookies/index.js';
export async function refreshToken(request, options) {
    const result = await verifyNextCookies(request.cookies, request.headers, options);
    const headers = {
        'Content-Type': 'application/json'
    };
    if (!result) {
        return new NextResponse(JSON.stringify({ idToken: null }), {
            status: 200,
            headers
        });
    }
    const response = new NextResponse(JSON.stringify({
        idToken: result.idToken,
        customToken: result.customToken
    }), {
        status: 200,
        headers
    });
    await appendAuthCookies(request.headers, response, result, options);
    return response;
}
