"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAppCheck = getAppCheck;
const credential_1 = require("../auth/credential");
const default_credential_js_1 = require("../auth/default-credential.js");
const token_generator_js_1 = require("../auth/token-generator.js");
const api_client_js_1 = require("./api-client.js");
const token_generator_js_2 = require("./token-generator.js");
const token_verifier_js_1 = require("./token-verifier.js");
class AppCheck {
    client;
    tokenGenerator;
    appCheckTokenVerifier;
    constructor(credential, tenantId) {
        this.client = new api_client_js_1.AppCheckApiClient(credential);
        this.tokenGenerator = new token_generator_js_2.AppCheckTokenGenerator((0, token_generator_js_1.cryptoSignerFromCredential)(credential, tenantId));
        this.appCheckTokenVerifier = new token_verifier_js_1.AppCheckTokenVerifier(credential);
    }
    createToken = (appId, options) => {
        return this.tokenGenerator
            .createCustomToken(appId, options)
            .then((customToken) => {
            return this.client.exchangeToken(customToken, appId);
        });
    };
    verifyToken = (appCheckToken, options) => {
        return this.appCheckTokenVerifier
            .verifyToken(appCheckToken, options)
            .then((decodedToken) => {
            return {
                appId: decodedToken.app_id,
                token: decodedToken
            };
        });
    };
}
function isAppCheckOptions(options) {
    const serviceAccount = options;
    return (!serviceAccount.privateKey ||
        !serviceAccount.projectId ||
        !serviceAccount.clientEmail);
}
function getAppCheck(serviceAccount, tenantId) {
    if (!isAppCheckOptions(serviceAccount)) {
        return new AppCheck(new credential_1.ServiceAccountCredential(serviceAccount), tenantId);
    }
    const options = serviceAccount;
    const credential = options.serviceAccount
        ? new credential_1.ServiceAccountCredential(options.serviceAccount)
        : (0, default_credential_js_1.getApplicationDefault)();
    return new AppCheck(credential, tenantId);
}
