export interface GoogleOAuthAccessToken {
    access_token: string;
    expires_in: number;
}
export interface Credential {
    getProjectId(): Promise<string>;
    getServiceAccountEmail(): Promise<string | null>;
    getAccessToken(forceRefresh?: boolean): Promise<FirebaseAccessToken>;
}
export interface ServiceAccount {
    projectId: string;
    privateKey: string;
    clientEmail: string;
}
export declare class ServiceAccountCredential implements Credential {
    readonly projectId: string;
    readonly privateKey: string;
    readonly clientEmail: string;
    constructor(serviceAccount: ServiceAccount);
    private fetchAccessToken;
    getProjectId(): Promise<string>;
    getServiceAccountEmail(): Promise<string>;
    private fetchAndCacheAccessToken;
    getAccessToken(forceRefresh: boolean): Promise<FirebaseAccessToken>;
    private createJwt;
}
export declare function getExplicitProjectId(): string | null;
export declare class ComputeEngineCredential implements Credential {
    private projectId?;
    private accountId?;
    private cachedToken?;
    constructor();
    getAccessToken(forceRefresh?: boolean): Promise<FirebaseAccessToken>;
    getIDToken(audience: string): Promise<string>;
    getProjectId(): Promise<string>;
    getServiceAccountEmail(): Promise<string>;
    private buildRequest;
}
export interface FirebaseAccessToken {
    accessToken: string;
    expirationTime: number;
}
export declare const getFirebaseAdminTokenProvider: (credential: Credential) => {
    getToken: (forceRefresh?: boolean) => Promise<FirebaseAccessToken>;
};
