import { JWTPayload, JWTVerifyResult } from 'jose';
import { DecodedIdToken } from '../types.js';
export interface CustomTokens {
    idToken: string;
    refreshToken: string;
    customToken: string;
}
export interface ParsedTokens {
    idToken: string;
    refreshToken: string;
    customToken?: string;
}
export interface VerifiedTokens {
    idToken: string;
    refreshToken: string;
    customToken?: string;
    decodedIdToken: DecodedIdToken;
}
export interface CustomJWTHeader {
    alg: 'HS256';
    typ: 'JWT';
}
export interface CustomJWTPayload extends JWTPayload {
    id_token: string;
    refresh_token: string;
    custom_token?: string;
}
export declare function createCustomSignature(tokens: ParsedTokens, key: string): Promise<string>;
export declare function verifyCustomSignature(tokens: ParsedTokens, signature: string, key: string): Promise<void>;
export declare function createCustomJWT(payload: CustomJWTPayload, secret: string): Promise<string>;
export declare function verifyCustomJWT(customJWT: string, secret: string): Promise<JWTVerifyResult<CustomJWTPayload>>;
