"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCustomSignature = createCustomSignature;
exports.verifyCustomSignature = verifyCustomSignature;
exports.createCustomJWT = createCustomJWT;
exports.verifyCustomJWT = verifyCustomJWT;
const jose_1 = require("jose");
const utils_js_1 = require("../utils.js");
async function createCustomSignature(tokens, key) {
    const jws = await new jose_1.FlattenedSign((0, utils_js_1.toUint8Array)(tokens.customToken
        ? `${tokens.idToken}.${tokens.refreshToken}.${tokens.customToken}`
        : `${tokens.idToken}.${tokens.refreshToken}`))
        .setProtectedHeader({ alg: 'HS256' })
        .sign((0, utils_js_1.toUint8Array)(key));
    return jws.signature;
}
async function verifyCustomSignature(tokens, signature, key) {
    if ((await createCustomSignature(tokens, key)) !== signature) {
        throw new jose_1.errors.JWSSignatureVerificationFailed('');
    }
}
async function createCustomJWT(payload, secret) {
    const jwt = new jose_1.SignJWT(payload);
    jwt.setProtectedHeader({ alg: 'HS256', typ: 'JWT' });
    return jwt.sign((0, utils_js_1.toUint8Array)(secret));
}
async function verifyCustomJWT(customJWT, secret) {
    return (0, jose_1.jwtVerify)(customJWT, (0, utils_js_1.toUint8Array)(secret));
}
