import { CreateRequest, UpdateRequest } from './auth-request-handler';
import { Credential, ServiceAccount } from './credential';
import { CustomTokens, ParsedTokens, VerifiedTokens } from './custom-token/index.js';
import { AuthError } from './error';
import { DecodedIdToken, VerifyOptions } from './types.js';
import { UserRecord } from './user-record.js';
export * from './types.js';
export * from './error.js';
interface CustomTokenToIdAndRefreshTokensOptions {
    tenantId?: string;
    appCheckToken?: string;
    referer?: string;
}
export declare function customTokenToIdAndRefreshTokens(customToken: string, firebaseApiKey: string, options: CustomTokenToIdAndRefreshTokensOptions): Promise<IdAndRefreshTokens>;
export declare function createAnonymousAccount(firebaseApiKey: string, options?: CreateAnonymousRequest): Promise<AnonymousTokens>;
export interface TokenRefreshOptions {
    apiKey: string;
    referer?: string;
}
export declare function isUserNotFoundError(error: unknown): error is AuthError;
export declare function isInvalidCredentialError(error: unknown): error is AuthError;
export declare function handleExpiredToken<T>(verifyIdToken: () => Promise<T>, onExpired: (e: AuthError) => Promise<T>, onError: (e: unknown) => Promise<T>, shouldExpireOnNoMatchingKidError: boolean): Promise<T>;
export interface IdAndRefreshTokens {
    idToken: string;
    refreshToken: string;
}
export interface CreateAnonymousRequest {
    tenantId?: string;
    appCheckToken?: string;
    referer?: string;
}
export interface Tokens {
    decodedToken: DecodedIdToken;
    token: string;
    customToken?: string;
}
export interface AnonymousTokens {
    idToken: string;
    refreshToken: string;
    localId: string;
}
export interface UsersList {
    users: UserRecord[];
    nextPageToken?: string;
}
export interface GetCustomIdAndRefreshTokensOptions {
    appCheckToken?: string;
    referer?: string;
    dynamicCustomClaimsKeys?: string[];
}
interface AuthOptions {
    credential: Credential;
    apiKey: string;
    tenantId?: string;
    serviceAccountId?: string;
    enableCustomToken?: boolean;
}
export type Auth = ReturnType<typeof getAuth>;
declare function getAuth(options: AuthOptions): {
    verifyAndRefreshExpiredIdToken: (parsedTokens: ParsedTokens, verifyOptions?: VerifyOptions & {
        onTokenRefresh?: (tokens: VerifiedTokens) => Promise<void>;
    }) => Promise<VerifiedTokens>;
    verifyIdToken: (idToken: string, options?: VerifyOptions) => Promise<DecodedIdToken>;
    createCustomToken: (uid: string, developerClaims?: {
        [key: string]: unknown;
    }) => Promise<string>;
    getCustomIdAndRefreshTokens: (idToken: string, customTokensOptions?: GetCustomIdAndRefreshTokensOptions) => Promise<CustomTokens>;
    handleTokenRefresh: (refreshToken: string, tokenRefreshOptions?: {
        referer?: string;
        enableCustomToken?: boolean;
    }) => Promise<VerifiedTokens>;
    deleteUser: (uid: string) => Promise<void>;
    setCustomUserClaims: (uid: string, customUserClaims: object | null) => Promise<void>;
    getUser: (uid: string) => Promise<UserRecord | null>;
    getUserByEmail: (email: string) => Promise<UserRecord>;
    updateUser: (uid: string, properties: UpdateRequest) => Promise<UserRecord>;
    createUser: (properties: CreateRequest) => Promise<UserRecord>;
    createAnonymousUser: (firebaseApiKey: string) => Promise<AnonymousTokens>;
    listUsers: (nextPageToken?: string, maxResults?: number) => Promise<UsersList>;
    createSessionCookie: (idToken: string, expiresInMs: number) => Promise<string>;
};
export interface FirebaseAuthOptions {
    serviceAccount?: ServiceAccount;
    apiKey: string;
    tenantId?: string;
    serviceAccountId?: string;
    enableCustomToken?: boolean;
}
export declare function getFirebaseAuth(options: FirebaseAuthOptions): Auth;
export declare function getFirebaseAuth(serviceAccount: ServiceAccount, apiKey: string, tenantId?: string): Auth;
