"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.customTokenToIdAndRefreshTokens = customTokenToIdAndRefreshTokens;
exports.createAnonymousAccount = createAnonymousAccount;
exports.isUserNotFoundError = isUserNotFoundError;
exports.isInvalidCredentialError = isInvalidCredentialError;
exports.handleExpiredToken = handleExpiredToken;
exports.getFirebaseAuth = getFirebaseAuth;
const index_js_1 = require("../debug/index.js");
const auth_request_handler_1 = require("./auth-request-handler");
const credential_1 = require("./credential");
const default_credential_js_1 = require("./default-credential.js");
const error_1 = require("./error");
const firebase_js_1 = require("./firebase.js");
const token_generator_js_1 = require("./token-generator.js");
const token_verifier_js_1 = require("./token-verifier.js");
const user_record_js_1 = require("./user-record.js");
const claims_1 = require("./claims");
__exportStar(require("./types.js"), exports);
__exportStar(require("./error.js"), exports);
const getCustomTokenEndpoint = (apiKey) => {
    if ((0, firebase_js_1.useEmulator)() && process.env.FIREBASE_AUTH_EMULATOR_HOST) {
        let protocol = 'http://';
        if (process.env.FIREBASE_AUTH_EMULATOR_HOST.startsWith('http://')) {
            protocol = '';
        }
        return `${protocol}${process.env
            .FIREBASE_AUTH_EMULATOR_HOST}/identitytoolkit.googleapis.com/v1/accounts:signInWithCustomToken?key=${apiKey}`;
    }
    return `https://identitytoolkit.googleapis.com/v1/accounts:signInWithCustomToken?key=${apiKey}`;
};
const getSignUpEndpoint = (apiKey) => {
    return `https://identitytoolkit.googleapis.com/v1/accounts:signUp?key=${apiKey}`;
};
const getRefreshTokenEndpoint = (apiKey) => {
    if ((0, firebase_js_1.useEmulator)() && process.env.FIREBASE_AUTH_EMULATOR_HOST) {
        let protocol = 'http://';
        if (process.env.FIREBASE_AUTH_EMULATOR_HOST.startsWith('http://')) {
            protocol = '';
        }
        return `${protocol}${process.env
            .FIREBASE_AUTH_EMULATOR_HOST}/securetoken.googleapis.com/v1/token?key=${apiKey}`;
    }
    return `https://securetoken.googleapis.com/v1/token?key=${apiKey}`;
};
async function customTokenToIdAndRefreshTokens(customToken, firebaseApiKey, options) {
    const headers = {
        'Content-Type': 'application/json',
        ...(options.referer ? { Referer: options.referer } : {})
    };
    const body = {
        token: customToken,
        returnSecureToken: true
    };
    if (options.appCheckToken) {
        headers['X-Firebase-AppCheck'] = options.appCheckToken;
    }
    if (options.tenantId) {
        body['tenantId'] = options.tenantId;
    }
    const refreshTokenResponse = await fetch(getCustomTokenEndpoint(firebaseApiKey), {
        method: 'POST',
        headers,
        body: JSON.stringify(body)
    });
    const refreshTokenJSON = (await refreshTokenResponse.json());
    if (!refreshTokenResponse.ok) {
        throw new Error(`Problem getting a refresh token: ${JSON.stringify(refreshTokenJSON)}`);
    }
    return {
        idToken: refreshTokenJSON.idToken,
        refreshToken: refreshTokenJSON.refreshToken
    };
}
async function createAnonymousAccount(firebaseApiKey, options = {}) {
    const headers = {
        'Content-Type': 'application/json',
        ...(options.referer ? { Referer: options.referer } : {})
    };
    const body = {
        returnSecureToken: true
    };
    if (options.appCheckToken) {
        headers['X-Firebase-AppCheck'] = options.appCheckToken;
    }
    if (options.tenantId) {
        body['tenantId'] = options.tenantId;
    }
    const createResponse = await fetch(getSignUpEndpoint(firebaseApiKey), {
        method: 'POST',
        headers,
        body: JSON.stringify(body)
    });
    return (await createResponse.json());
}
const isUserNotFoundResponse = (data) => {
    return (data?.error?.code === 400 &&
        data?.error?.message === 'USER_NOT_FOUND');
};
const refreshExpiredIdToken = async (refreshToken, options) => {
    const response = await fetch(getRefreshTokenEndpoint(options.apiKey), {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
            ...(options.referer ? { Referer: options.referer } : {})
        },
        body: `grant_type=refresh_token&refresh_token=${refreshToken}`
    });
    if (!response.ok) {
        const data = await response.json();
        const errorMessage = `Error fetching access token: ${JSON.stringify(data.error)} ${data.error_description ? `(${data.error_description})` : ''}`;
        if (isUserNotFoundResponse(data)) {
            throw new error_1.AuthError(error_1.AuthErrorCode.USER_NOT_FOUND);
        }
        throw new error_1.AuthError(error_1.AuthErrorCode.INVALID_CREDENTIAL, errorMessage);
    }
    const data = await response.json();
    return {
        idToken: data.id_token,
        refreshToken: data.refresh_token
    };
};
function isUserNotFoundError(error) {
    return error?.code === error_1.AuthErrorCode.USER_NOT_FOUND;
}
function isInvalidCredentialError(error) {
    return error?.code === error_1.AuthErrorCode.INVALID_CREDENTIAL;
}
async function handleVerifyTokenError(e, onExpired, onError) {
    try {
        return await onExpired(e);
    }
    catch (e) {
        return onError(e);
    }
}
async function handleExpiredToken(verifyIdToken, onExpired, onError, shouldExpireOnNoMatchingKidError) {
    try {
        return await verifyIdToken();
    }
    catch (e) {
        switch (e?.code) {
            case error_1.AuthErrorCode.NO_MATCHING_KID:
                if (shouldExpireOnNoMatchingKidError) {
                    return handleVerifyTokenError(e, async (e) => {
                        const result = await onExpired(e);
                        (0, index_js_1.debug)('experimental_refresh_on_expired_kid: Successfully refreshed token after kid has expired');
                        return result;
                    }, (e) => {
                        (0, index_js_1.debug)('experimental_refresh_on_expired_kid: Error when trying to refresh token after kid has expired', { message: e?.message, stack: e?.stack });
                        return onError(e);
                    });
                }
                return onError(e);
            case error_1.AuthErrorCode.TOKEN_EXPIRED:
                return handleVerifyTokenError(e, onExpired, onError);
            default:
                return onError(e);
        }
    }
}
const DEFAULT_VERIFY_OPTIONS = { referer: '' };
function getAuth(options) {
    const credential = options.credential ?? (0, default_credential_js_1.getApplicationDefault)();
    const tenantId = options.tenantId;
    const authRequestHandler = new auth_request_handler_1.AuthRequestHandler(credential, {
        tenantId
    });
    const tokenGenerator = (0, token_generator_js_1.createFirebaseTokenGenerator)(credential, tenantId);
    const handleTokenRefresh = async (refreshToken, tokenRefreshOptions = {}) => {
        const { idToken, refreshToken: newRefreshToken } = await refreshExpiredIdToken(refreshToken, {
            apiKey: options.apiKey,
            referer: tokenRefreshOptions.referer
        });
        const decodedIdToken = await verifyIdToken(idToken, {
            referer: tokenRefreshOptions.referer
        });
        if (!tokenRefreshOptions.enableCustomToken) {
            return {
                decodedIdToken,
                idToken,
                refreshToken: newRefreshToken
            };
        }
        const customToken = await createCustomToken(decodedIdToken.uid, {
            email_verified: decodedIdToken.email_verified,
            source_sign_in_provider: decodedIdToken.firebase.sign_in_provider
        });
        return {
            decodedIdToken,
            idToken,
            refreshToken: newRefreshToken,
            customToken
        };
    };
    async function createSessionCookie(idToken, expiresInMs) {
        if (tenantId) {
            await verifyIdToken(idToken);
        }
        return authRequestHandler.createSessionCookie(idToken, expiresInMs);
    }
    async function getUser(uid) {
        return authRequestHandler.getAccountInfoByUid(uid).then((response) => {
            return response.users?.length ? new user_record_js_1.UserRecord(response.users[0]) : null;
        });
    }
    async function listUsers(nextPageToken, maxResults) {
        return authRequestHandler
            .listUsers(nextPageToken, maxResults)
            .then((response) => {
            const result = {
                users: response.users.map((user) => new user_record_js_1.UserRecord(user))
            };
            if (response.nextPageToken) {
                result.nextPageToken = response.nextPageToken;
            }
            return result;
        });
    }
    async function getUserByEmail(email) {
        return authRequestHandler.getAccountInfoByEmail(email).then((response) => {
            if (!response.users || !response.users.length) {
                throw new error_1.AuthError(error_1.AuthErrorCode.USER_NOT_FOUND);
            }
            return new user_record_js_1.UserRecord(response.users[0]);
        });
    }
    async function verifyDecodedJWTNotRevokedOrDisabled(decodedIdToken) {
        return getUser(decodedIdToken.sub).then((user) => {
            if (!user) {
                throw new error_1.AuthError(error_1.AuthErrorCode.USER_NOT_FOUND);
            }
            if (user.disabled) {
                throw new error_1.AuthError(error_1.AuthErrorCode.USER_DISABLED);
            }
            if (user.tokensValidAfterTime) {
                const authTimeUtc = decodedIdToken.auth_time * 1000;
                const validSinceUtc = new Date(user.tokensValidAfterTime).getTime();
                if (authTimeUtc < validSinceUtc) {
                    throw new error_1.AuthError(error_1.AuthErrorCode.TOKEN_REVOKED);
                }
            }
            return decodedIdToken;
        });
    }
    async function verifyIdToken(idToken, options = DEFAULT_VERIFY_OPTIONS) {
        const projectId = await credential.getProjectId();
        const idTokenVerifier = (0, token_verifier_js_1.createIdTokenVerifier)(projectId, tenantId);
        const decodedIdToken = await idTokenVerifier.verifyJWT(idToken, options);
        const checkRevoked = options.checkRevoked ?? false;
        if (checkRevoked) {
            return verifyDecodedJWTNotRevokedOrDisabled(decodedIdToken);
        }
        return decodedIdToken;
    }
    async function verifyAndRefreshExpiredIdToken(parsedTokens, verifyOptions = DEFAULT_VERIFY_OPTIONS) {
        return await handleExpiredToken(async () => {
            const decodedIdToken = await verifyIdToken(parsedTokens.idToken, verifyOptions);
            return {
                idToken: parsedTokens.idToken,
                decodedIdToken,
                refreshToken: parsedTokens.refreshToken,
                customToken: parsedTokens.customToken
            };
        }, async () => {
            if (parsedTokens.refreshToken) {
                const result = await handleTokenRefresh(parsedTokens.refreshToken, {
                    referer: verifyOptions.referer,
                    enableCustomToken: options.enableCustomToken
                });
                await verifyOptions.onTokenRefresh?.(result);
                return result;
            }
            throw new error_1.InvalidTokenError(error_1.InvalidTokenReason.MISSING_REFRESH_TOKEN);
        }, async (e) => {
            if (e instanceof error_1.AuthError &&
                e.code === error_1.AuthErrorCode.NO_MATCHING_KID) {
                throw error_1.InvalidTokenError.fromError(e, error_1.InvalidTokenReason.INVALID_KID);
            }
            throw error_1.InvalidTokenError.fromError(e, error_1.InvalidTokenReason.INVALID_CREDENTIALS);
        }, verifyOptions.experimental_enableTokenRefreshOnExpiredKidHeader ?? false);
    }
    function createCustomToken(uid, developerClaims) {
        return tokenGenerator.createCustomToken(uid, developerClaims);
    }
    async function getCustomIdAndRefreshTokens(idToken, customTokensOptions = DEFAULT_VERIFY_OPTIONS) {
        const decodedToken = await verifyIdToken(idToken, {
            referer: customTokensOptions.referer
        });
        const customClaims = (0, claims_1.filterStandardClaims)(decodedToken);
        if (customTokensOptions.dynamicCustomClaimsKeys?.length) {
            customTokensOptions.dynamicCustomClaimsKeys.forEach((key) => {
                delete customClaims[key];
            });
        }
        const customToken = await createCustomToken(decodedToken.uid, {
            ...customClaims,
            email_verified: decodedToken.email_verified,
            source_sign_in_provider: decodedToken.firebase.sign_in_provider
        });
        (0, index_js_1.debug)('Generated custom token based on provided idToken', { customToken });
        const idAndRefreshTokens = await customTokenToIdAndRefreshTokens(customToken, options.apiKey, {
            tenantId: options.tenantId,
            appCheckToken: customTokensOptions.appCheckToken,
            referer: customTokensOptions.referer
        });
        return {
            ...idAndRefreshTokens,
            customToken
        };
    }
    async function deleteUser(uid) {
        await authRequestHandler.deleteAccount(uid);
    }
    async function setCustomUserClaims(uid, customUserClaims) {
        await authRequestHandler.setCustomUserClaims(uid, customUserClaims);
    }
    async function createUser(properties) {
        return authRequestHandler
            .createNewAccount(properties)
            .then((uid) => getUser(uid))
            .then((user) => {
            if (!user) {
                throw new error_1.AuthError(error_1.AuthErrorCode.INTERNAL_ERROR, 'Could not get recently created user from database. Most likely it was deleted.');
            }
            return user;
        });
    }
    async function createAnonymousUser(firebaseApiKey) {
        return createAnonymousAccount(firebaseApiKey);
    }
    async function updateUser(uid, properties) {
        return authRequestHandler
            .updateExistingAccount(uid, properties)
            .then((existingUid) => getUser(existingUid))
            .then((user) => {
            if (!user) {
                throw new error_1.AuthError(error_1.AuthErrorCode.INTERNAL_ERROR, 'Could not get recently updated user from database. Most likely it was deleted.');
            }
            return user;
        });
    }
    return {
        verifyAndRefreshExpiredIdToken,
        verifyIdToken,
        createCustomToken,
        getCustomIdAndRefreshTokens,
        handleTokenRefresh,
        deleteUser,
        setCustomUserClaims,
        getUser,
        getUserByEmail,
        updateUser,
        createUser,
        createAnonymousUser,
        listUsers,
        createSessionCookie
    };
}
function isFirebaseAuthOptions(options) {
    const serviceAccount = options;
    return (!serviceAccount.privateKey ||
        !serviceAccount.projectId ||
        !serviceAccount.clientEmail);
}
function getFirebaseAuth(serviceAccount, apiKey, tenantId) {
    if (!isFirebaseAuthOptions(serviceAccount)) {
        const credential = new credential_1.ServiceAccountCredential(serviceAccount);
        return getAuth({ credential, apiKey: apiKey, tenantId });
    }
    const options = serviceAccount;
    return getAuth({
        credential: options.serviceAccount
            ? new credential_1.ServiceAccountCredential(options.serviceAccount)
            : (0, default_credential_js_1.getApplicationDefault)(),
        apiKey: options.apiKey,
        tenantId: options.tenantId,
        serviceAccountId: options.serviceAccountId,
        enableCustomToken: options.enableCustomToken
    });
}
