"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sign = sign;
exports.signBlob = signBlob;
const jose_1 = require("jose");
const error_js_1 = require("../error.js");
const utils_js_1 = require("../utils.js");
const verify_js_1 = require("./verify.js");
async function sign({ payload, privateKey, keyId }) {
    let key;
    try {
        key = await (0, jose_1.importPKCS8)(privateKey, verify_js_1.ALGORITHM_RS256);
    }
    catch (e) {
        throw error_js_1.AuthError.fromError(e, error_js_1.AuthErrorCode.INVALID_ARGUMENT, 'It looks like the value provided for `serviceAccount.privateKey` is incorrectly formatted. Please double-check if private key has correct format. See https://github.com/awinogrodzki/next-firebase-auth-edge/issues/246#issuecomment-2321559620 for details');
    }
    return new jose_1.SignJWT(payload)
        .setProtectedHeader({ alg: verify_js_1.ALGORITHM_RS256, kid: keyId })
        .sign(key);
}
function formatBase64(value) {
    return value.replace(/\//g, '_').replace(/\+/g, '-').replace(/=+$/, '');
}
function encodeSegment(segment) {
    const value = JSON.stringify(segment);
    return formatBase64(jose_1.base64url.encode(value));
}
async function signBlob({ payload, serviceAccountId, accessToken }) {
    const url = `https://iamcredentials.googleapis.com/v1/projects/-/serviceAccounts/${serviceAccountId}:signBlob`;
    const header = {
        alg: verify_js_1.ALGORITHM_RS256,
        typ: 'JWT'
    };
    const token = `${encodeSegment(header)}.${encodeSegment(payload)}`;
    const request = {
        method: 'POST',
        headers: {
            Authorization: `Bearer ${accessToken}`
        },
        body: JSON.stringify({ payload: jose_1.base64url.encode(token) })
    };
    const response = await (0, utils_js_1.fetchAny)(url, request);
    const blob = await response.blob();
    const key = await blob.text();
    const { signedBlob } = JSON.parse(key);
    return `${token}.${formatBase64(signedBlob)}`;
}
