"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ALGORITHM_RS256 = void 0;
exports.getPublicCryptoKey = getPublicCryptoKey;
exports.verify = verify;
const jose_1 = require("jose");
const firebase_js_1 = require("../firebase.js");
exports.ALGORITHM_RS256 = 'RS256';
const keyMap = new Map();
async function importPublicCryptoKey(publicKey) {
    if (publicKey.startsWith('-----BEGIN CERTIFICATE-----')) {
        return (0, jose_1.importX509)(publicKey, exports.ALGORITHM_RS256);
    }
    return (0, jose_1.importSPKI)(publicKey, exports.ALGORITHM_RS256);
}
async function getPublicCryptoKey(publicKey) {
    const cachedKey = keyMap.get(publicKey);
    if (cachedKey) {
        return cachedKey;
    }
    const key = await importPublicCryptoKey(publicKey);
    keyMap.set(publicKey, key);
    return key;
}
async function verify(jwtString, getPublicKey, options) {
    const currentDate = options.currentDate ?? new Date();
    const currentTimestamp = currentDate.getTime() / 1000;
    const payload = (0, jose_1.decodeJwt)(jwtString);
    if (!(0, firebase_js_1.useEmulator)()) {
        const { payload } = await (0, jose_1.jwtVerify)(jwtString, await getPublicKey(), {
            currentDate
        });
        return payload;
    }
    if (typeof payload.nbf !== 'undefined') {
        if (typeof payload.nbf !== 'number') {
            throw new jose_1.errors.JWTInvalid('invalid nbf value');
        }
        if (payload.nbf > currentTimestamp) {
            throw new jose_1.errors.JWTExpired('jwt not active: ' + new Date(payload.nbf * 1000).toISOString(), payload);
        }
    }
    if (typeof payload.exp !== 'undefined') {
        if (typeof payload.exp !== 'number') {
            throw new jose_1.errors.JWTInvalid('invalid exp value');
        }
        if (currentTimestamp >= payload.exp) {
            throw new jose_1.errors.JWTExpired('token expired: ' + new Date(payload.exp * 1000).toISOString(), payload);
        }
    }
    return payload;
}
