"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RotatingCredential = void 0;
const jose_1 = require("jose");
const index_js_1 = require("./custom-token/index.js");
class RotatingCredential {
    keys;
    constructor(keys) {
        this.keys = keys;
    }
    async sign(payload) {
        return (0, index_js_1.createCustomJWT)(payload, this.keys[0]);
    }
    async createSignature(tokens) {
        return (0, index_js_1.createCustomSignature)(tokens, this.keys[0]);
    }
    async verify(customJWT) {
        for (const key of this.keys) {
            try {
                const result = await (0, index_js_1.verifyCustomJWT)(customJWT, key);
                return result.payload;
            }
            catch (e) {
                if (e instanceof jose_1.errors.JWSSignatureVerificationFailed ||
                    e instanceof jose_1.errors.JWSInvalid) {
                    continue;
                }
                throw e;
            }
        }
        throw new jose_1.errors.JWSSignatureVerificationFailed('Custom JWT could not be verified against any of the provided keys');
    }
    async verifySignature(tokens, signature) {
        for (const key of this.keys) {
            try {
                return await (0, index_js_1.verifyCustomSignature)(tokens, signature, key);
            }
            catch (e) {
                if (e instanceof jose_1.errors.JWSSignatureVerificationFailed ||
                    e instanceof jose_1.errors.JWSInvalid) {
                    continue;
                }
                throw e;
            }
        }
        throw new jose_1.errors.JWSSignatureVerificationFailed('Custom tokens signature could not be verified against any of the provided keys');
    }
}
exports.RotatingCredential = RotatingCredential;
