"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatString = formatString;
exports.fetchJson = fetchJson;
exports.fetchText = fetchText;
exports.fetchAny = fetchAny;
exports.mapJwtPayloadToDecodedIdToken = mapJwtPayloadToDecodedIdToken;
exports.addReadonlyGetter = addReadonlyGetter;
exports.deepCopy = deepCopy;
exports.deepExtend = deepExtend;
exports.toUint8Array = toUint8Array;
function formatString(str, params) {
    let formatted = str;
    Object.keys(params || {}).forEach((key) => {
        formatted = formatted.replace(new RegExp('{' + key + '}', 'g'), params[key]);
    });
    return formatted;
}
async function getDetailFromResponse(response) {
    const json = await response.json();
    if (!json) {
        return 'Missing error payload';
    }
    let detail = typeof json.error === 'string'
        ? json.error
        : (json.error?.message ?? 'Missing error payload');
    if (json.error_description) {
        detail += ' (' + json.error_description + ')';
    }
    return detail;
}
async function fetchJson(url, init) {
    return (await fetchAny(url, init)).json();
}
async function fetchText(url, init) {
    return (await fetchAny(url, init)).text();
}
async function fetchAny(url, init) {
    const response = await fetch(url, init);
    if (!response.ok) {
        throw new Error(await getDetailFromResponse(response));
    }
    return response;
}
function mapJwtPayloadToDecodedIdToken(payload) {
    const decodedIdToken = payload;
    decodedIdToken.uid = decodedIdToken.sub;
    return decodedIdToken;
}
function addReadonlyGetter(obj, prop, value) {
    Object.defineProperty(obj, prop, {
        value,
        writable: false,
        enumerable: true
    });
}
function deepCopy(value) {
    return deepExtend(undefined, value);
}
function deepExtend(target, source) {
    if (!(source instanceof Object)) {
        return source;
    }
    switch (source.constructor) {
        case Date: {
            const dateValue = source;
            return new Date(dateValue.getTime());
        }
        case Object:
            if (target === undefined) {
                target = {};
            }
            break;
        case Array:
            target = [];
            break;
        default:
            return source;
    }
    for (const prop in source) {
        if (!Object.prototype.hasOwnProperty.call(source, prop)) {
            continue;
        }
        const objectTarget = target;
        objectTarget[prop] = deepExtend(objectTarget[prop], objectTarget[prop]);
    }
    return target;
}
const encoder = new TextEncoder();
function toUint8Array(key) {
    return encoder.encode(key);
}
