"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isObject = isObject;
exports.isArray = isArray;
exports.isNonNullObject = isNonNullObject;
exports.isEmail = isEmail;
exports.isURL = isURL;
function isObject(value) {
    return typeof value === 'object' && !isArray(value);
}
function isArray(value) {
    return Array.isArray(value);
}
function isNonNullObject(value) {
    return isObject(value) && value !== null;
}
function isEmail(email) {
    if (typeof email !== 'string') {
        return false;
    }
    const re = /^[^@]+@[^@]+$/;
    return re.test(email);
}
function isURL(urlStr) {
    if (typeof urlStr !== 'string') {
        return false;
    }
    const re = /[^a-z0-9:/?#[\]@!$&'()*+,;=.\-_~%]/i;
    if (re.test(urlStr)) {
        return false;
    }
    try {
        const uri = new URL(urlStr);
        const scheme = uri.protocol;
        const hostname = uri.hostname;
        const pathname = uri.pathname;
        if (scheme !== 'http:' && scheme !== 'https:') {
            return false;
        }
        if (!hostname ||
            !/^[a-zA-Z0-9]+[\w-]*([.]?[a-zA-Z0-9]+[\w-]*)*$/.test(hostname)) {
            return false;
        }
        const pathnameRe = /^(\/[\w\-.~!$'()*+,;=:@%]+)*\/?$/;
        if (pathname && pathname !== '/' && !pathnameRe.test(pathname)) {
            return false;
        }
    }
    catch {
        return false;
    }
    return true;
}
