"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshApiResponseCookies = refreshApiResponseCookies;
exports.appendAuthCookiesApi = appendAuthCookiesApi;
exports.refreshApiCookies = refreshApiCookies;
const index_js_1 = require("../auth/index.js");
const AuthCookies_js_1 = require("./cookies/AuthCookies.js");
const ObjectCookiesProvider_js_1 = require("./cookies/parser/ObjectCookiesProvider.js");
const tokens_js_1 = require("./tokens.js");
async function refreshApiResponseCookies(request, response, options) {
    const tokens = await refreshApiCookies(request.cookies, request.headers, options);
    await appendAuthCookiesApi(request.cookies, response, tokens, options);
    return response;
}
async function appendAuthCookiesApi(cookies, response, tokens, options) {
    const authCookies = new AuthCookies_js_1.AuthCookies(new ObjectCookiesProvider_js_1.ObjectCookiesProvider(cookies), options);
    await authCookies.setAuthNextApiResponseHeaders(tokens, response);
}
async function refreshApiCookies(cookies, headers, options) {
    const referer = headers['referer'] ?? '';
    const tokens = await (0, tokens_js_1.getCookiesTokens)(cookies, options);
    const { handleTokenRefresh } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const tokenRefreshResult = await handleTokenRefresh(tokens.refreshToken, {
        referer,
        enableCustomToken: options.enableCustomToken
    });
    return {
        customToken: tokenRefreshResult.customToken,
        idToken: tokenRefreshResult.idToken,
        refreshToken: tokenRefreshResult.refreshToken,
        decodedIdToken: tokenRefreshResult.decodedIdToken
    };
}
