"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultipleCookieBuilder = void 0;
const rotating_credential_js_1 = require("../../../auth/rotating-credential.js");
class MultipleCookieBuilder {
    cookieName;
    credential;
    constructor(cookieName, signatureKeys) {
        this.cookieName = cookieName;
        this.credential = new rotating_credential_js_1.RotatingCredential(signatureKeys);
    }
    async buildCookies(tokens) {
        const signature = await this.credential.createSignature(tokens);
        if (!tokens.customToken) {
            return [
                {
                    name: `${this.cookieName}.id`,
                    value: tokens.idToken
                },
                {
                    name: `${this.cookieName}.refresh`,
                    value: tokens.refreshToken
                },
                {
                    name: `${this.cookieName}.sig`,
                    value: signature
                }
            ];
        }
        return [
            {
                name: `${this.cookieName}.id`,
                value: tokens.idToken
            },
            {
                name: `${this.cookieName}.refresh`,
                value: tokens.refreshToken
            },
            {
                name: `${this.cookieName}.custom`,
                value: tokens.customToken
            },
            {
                name: `${this.cookieName}.sig`,
                value: signature
            }
        ];
    }
}
exports.MultipleCookieBuilder = MultipleCookieBuilder;
