"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CookieExpirationFactory = void 0;
const CookieParserFactory_js_1 = require("../parser/CookieParserFactory.js");
const HeadersCookieSetter_js_1 = require("../setter/HeadersCookieSetter.js");
const CombinedCookieExpiration_js_1 = require("./CombinedCookieExpiration.js");
const MultipleCookieExpiration_js_1 = require("./MultipleCookieExpiration.js");
const SingleCookieExpiration_js_1 = require("./SingleCookieExpiration.js");
class CookieExpirationFactory {
    static fromSetter(setter, provider, cookieName) {
        const singleCookie = provider.get(cookieName);
        const hasEnabledMultipleCookies = CookieParserFactory_js_1.CookieParserFactory.hasMultipleCookies(provider, cookieName);
        const hasEnabledLegacyMultipleCookies = CookieParserFactory_js_1.CookieParserFactory.hasLegacyMultipleCookies(provider, cookieName);
        if (singleCookie &&
            (hasEnabledMultipleCookies || hasEnabledLegacyMultipleCookies)) {
            return new CombinedCookieExpiration_js_1.CombinedCookieExpiration(new MultipleCookieExpiration_js_1.MultipleCookieExpiration(cookieName, setter), new SingleCookieExpiration_js_1.SingleCookieExpiration(cookieName, setter));
        }
        if (hasEnabledMultipleCookies) {
            return new MultipleCookieExpiration_js_1.MultipleCookieExpiration(cookieName, setter);
        }
        return new SingleCookieExpiration_js_1.SingleCookieExpiration(cookieName, setter);
    }
    static fromHeaders(headers, provider, cookieName) {
        const setter = new HeadersCookieSetter_js_1.HeadersCookieSetter(headers);
        return CookieExpirationFactory.fromSetter(setter, provider, cookieName);
    }
}
exports.CookieExpirationFactory = CookieExpirationFactory;
