"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendAuthCookies = appendAuthCookies;
exports.setAuthCookies = setAuthCookies;
exports.removeServerCookies = removeServerCookies;
exports.removeCookies = removeCookies;
exports.removeAuthCookies = removeAuthCookies;
exports.verifyApiCookies = verifyApiCookies;
exports.verifyNextCookies = verifyNextCookies;
exports.refreshNextCookies = refreshNextCookies;
exports.refreshCredentials = refreshCredentials;
exports.refreshNextResponseCookiesWithToken = refreshNextResponseCookiesWithToken;
exports.refreshCookiesWithIdToken = refreshCookiesWithIdToken;
exports.refreshNextResponseCookies = refreshNextResponseCookies;
exports.refreshServerCookies = refreshServerCookies;
const server_1 = require("next/server");
const index_js_1 = require("../../auth/index.js");
const index_js_2 = require("../../debug/index.js");
const tokens_js_1 = require("../tokens.js");
const utils_js_1 = require("../utils.js");
const AuthCookies_js_1 = require("./AuthCookies.js");
const RequestCookiesProvider_js_1 = require("./parser/RequestCookiesProvider.js");
const CookieExpirationFactory_js_1 = require("./expiration/CookieExpirationFactory.js");
const CookieRemoverFactory_js_1 = require("./remover/CookieRemoverFactory.js");
async function appendAuthCookies(headers, response, tokens, options) {
    (0, index_js_2.debug)('Updating response headers with authenticated cookies');
    const authCookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthHeaders(tokens, response.headers);
}
async function setAuthCookies(headers, options) {
    const { getCustomIdAndRefreshTokens } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const authHeader = options.authorizationHeaderName ?? 'Authorization';
    const token = headers.get(authHeader)?.split(' ')[1] ?? '';
    if (!token) {
        const response = new server_1.NextResponse(JSON.stringify({ success: false, message: 'Missing token' }), {
            status: 400,
            headers: { 'content-type': 'application/json' }
        });
        return response;
    }
    const appCheckToken = headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = (0, utils_js_1.getReferer)(headers) ?? '';
    const customTokens = await getCustomIdAndRefreshTokens(token, {
        appCheckToken,
        referer,
        dynamicCustomClaimsKeys: options.dynamicCustomClaimsKeys
    });
    (0, index_js_2.debug)('Successfully generated custom tokens');
    const response = new server_1.NextResponse(JSON.stringify({ success: true }), {
        status: 200,
        headers: { 'content-type': 'application/json' }
    });
    await appendAuthCookies(headers, response, customTokens, options);
    return response;
}
function removeServerCookies(cookies, options) {
    const remover = CookieRemoverFactory_js_1.CookieRemoverFactory.fromRequestCookies(cookies, RequestCookiesProvider_js_1.RequestCookiesProvider.fromRequestCookies(cookies), options.cookieName);
    return remover.removeCookies();
}
function removeCookies(headers, response, options) {
    const expiration = CookieExpirationFactory_js_1.CookieExpirationFactory.fromHeaders(response.headers, RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(headers), options.cookieName);
    return expiration.expireCookies(options.cookieSerializeOptions);
}
function removeAuthCookies(headers, options) {
    const response = new server_1.NextResponse(JSON.stringify({ success: true }), {
        status: 200,
        headers: { 'content-type': 'application/json' }
    });
    removeCookies(headers, response, options);
    (0, index_js_2.debug)('Updating response with empty authentication cookie headers', {
        cookieName: options.cookieName
    });
    return response;
}
async function verifyApiCookies(cookies, headers, options) {
    const tokens = await (0, tokens_js_1.getCookiesTokens)(cookies, options);
    const { verifyAndRefreshExpiredIdToken } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const verifyTokenResult = await verifyAndRefreshExpiredIdToken(tokens, {
        referer: headers.referer ?? ''
    });
    return verifyTokenResult;
}
async function verifyNextCookies(cookies, headers, options) {
    const { verifyAndRefreshExpiredIdToken } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId,
        enableCustomToken: options.enableCustomToken
    });
    const referer = (0, utils_js_1.getReferer)(headers) ?? '';
    const tokens = await (0, tokens_js_1.getRequestCookiesTokens)(cookies, options);
    const verifyTokenResult = await verifyAndRefreshExpiredIdToken(tokens, {
        referer
    });
    return verifyTokenResult;
}
async function refreshNextCookies(cookies, headers, options) {
    const { handleTokenRefresh } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const referer = (0, utils_js_1.getReferer)(headers) ?? '';
    const tokens = await (0, tokens_js_1.getRequestCookiesTokens)(cookies, options);
    const tokenRefreshResult = await handleTokenRefresh(tokens.refreshToken, {
        referer,
        enableCustomToken: options.enableCustomToken
    });
    return {
        idToken: tokenRefreshResult.idToken,
        refreshToken: tokenRefreshResult.refreshToken,
        customToken: tokenRefreshResult.customToken,
        decodedIdToken: tokenRefreshResult.decodedIdToken
    };
}
async function refreshCredentials(request, options, responseFactory) {
    const tokens = await refreshNextCookies(request.cookies, request.headers, options);
    const cookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(request.headers), options);
    await cookies.setAuthCookies(tokens, request.cookies);
    const responseOrPromise = responseFactory({
        headers: request.headers,
        tokens
    });
    const response = responseOrPromise instanceof Promise
        ? await responseOrPromise
        : responseOrPromise;
    await cookies.setAuthHeaders(tokens, response.headers);
    return response;
}
async function refreshNextResponseCookiesWithToken(idToken, request, response, options) {
    const appCheckToken = request.headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = (0, utils_js_1.getReferer)(request.headers) ?? '';
    const { getCustomIdAndRefreshTokens } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const customTokens = await getCustomIdAndRefreshTokens(idToken, {
        appCheckToken,
        referer
    });
    await appendAuthCookies(request.headers, response, customTokens, options);
    return response;
}
async function refreshCookiesWithIdToken(idToken, headers, cookies, options) {
    const appCheckToken = headers.get('X-Firebase-AppCheck') ?? undefined;
    const referer = (0, utils_js_1.getReferer)(headers) ?? '';
    const { getCustomIdAndRefreshTokens } = (0, index_js_1.getFirebaseAuth)({
        serviceAccount: options.serviceAccount,
        apiKey: options.apiKey,
        tenantId: options.tenantId
    });
    const customTokens = await getCustomIdAndRefreshTokens(idToken, {
        appCheckToken,
        referer
    });
    const authCookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthCookies(customTokens, cookies);
}
async function refreshNextResponseCookies(request, response, options) {
    const customTokens = await refreshNextCookies(request.cookies, request.headers, options);
    await appendAuthCookies(request.headers, response, customTokens, options);
    return response;
}
async function refreshServerCookies(cookies, headers, options) {
    const customTokens = await refreshNextCookies(cookies, headers, options);
    const authCookies = new AuthCookies_js_1.AuthCookies(RequestCookiesProvider_js_1.RequestCookiesProvider.fromHeaders(headers), options);
    await authCookies.setAuthCookies(customTokens, cookies);
    await authCookies.setAuthHeaders(customTokens, headers);
}
__exportStar(require("./types.js"), exports);
