"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MultipleCookiesParser = void 0;
const jose_1 = require("jose");
const error_js_1 = require("../../../auth/error.js");
const rotating_credential_js_1 = require("../../../auth/rotating-credential.js");
class MultipleCookiesParser {
    cookies;
    cookieName;
    signatureKeys;
    constructor(cookies, cookieName, signatureKeys) {
        this.cookies = cookies;
        this.cookieName = cookieName;
        this.signatureKeys = signatureKeys;
    }
    async parseCookies() {
        const idTokenCookie = this.cookies.get(`${this.cookieName}.id`);
        const refreshTokenCookie = this.cookies.get(`${this.cookieName}.refresh`);
        const customTokenCookie = this.cookies.get(`${this.cookieName}.custom`);
        const signatureCookie = this.cookies.get(`${this.cookieName}.sig`);
        if (![idTokenCookie, refreshTokenCookie, signatureCookie].every(Boolean)) {
            throw new error_js_1.InvalidTokenError(error_js_1.InvalidTokenReason.MISSING_CREDENTIALS);
        }
        const signature = signatureCookie;
        const customTokens = {
            idToken: idTokenCookie,
            refreshToken: refreshTokenCookie,
            customToken: customTokenCookie
        };
        const credential = new rotating_credential_js_1.RotatingCredential(this.signatureKeys);
        try {
            await credential.verifySignature(customTokens, signature);
            return customTokens;
        }
        catch (e) {
            if (e instanceof jose_1.errors.JWSSignatureVerificationFailed) {
                throw new error_js_1.InvalidTokenError(error_js_1.InvalidTokenReason.INVALID_SIGNATURE);
            }
            throw e;
        }
    }
}
exports.MultipleCookiesParser = MultipleCookiesParser;
