"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SingleCookieParser = void 0;
const jose_1 = require("jose");
const error_js_1 = require("../../../auth/error.js");
const rotating_credential_js_1 = require("../../../auth/rotating-credential.js");
class SingleCookieParser {
    cookies;
    cookieName;
    signatureKeys;
    constructor(cookies, cookieName, signatureKeys) {
        this.cookies = cookies;
        this.cookieName = cookieName;
        this.signatureKeys = signatureKeys;
    }
    async parseCookies() {
        const jwtCookie = this.cookies.get(this.cookieName);
        if (!jwtCookie) {
            throw new error_js_1.InvalidTokenError(error_js_1.InvalidTokenReason.MISSING_CREDENTIALS);
        }
        const credential = new rotating_credential_js_1.RotatingCredential(this.signatureKeys);
        try {
            const result = await credential.verify(jwtCookie);
            return {
                idToken: result.id_token,
                refreshToken: result.refresh_token,
                customToken: result.custom_token
            };
        }
        catch (e) {
            if (e instanceof jose_1.errors.JWSSignatureVerificationFailed) {
                throw new error_js_1.InvalidTokenError(error_js_1.InvalidTokenReason.INVALID_SIGNATURE);
            }
            throw e;
        }
    }
}
exports.SingleCookieParser = SingleCookieParser;
