"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CookieRemoverFactory = void 0;
const CookieParserFactory_js_1 = require("../parser/CookieParserFactory.js");
const CombinedCookieRemover_1 = require("./CombinedCookieRemover");
const MultipleCookieRemover_1 = require("./MultipleCookieRemover");
const SingleCookieRemover_1 = require("./SingleCookieRemover");
class CookieRemoverFactory {
    static fromRequestCookies(cookies, provider, cookieName) {
        const singleCookie = provider.get(cookieName);
        const hasEnabledMultipleCookies = CookieParserFactory_js_1.CookieParserFactory.hasMultipleCookies(provider, cookieName);
        const hasEnabledLegacyMultipleCookies = CookieParserFactory_js_1.CookieParserFactory.hasLegacyMultipleCookies(provider, cookieName);
        if (singleCookie &&
            (hasEnabledMultipleCookies || hasEnabledLegacyMultipleCookies)) {
            return new CombinedCookieRemover_1.CombinedCookieRemover(new MultipleCookieRemover_1.MultipleCookieRemover(cookieName, cookies), new SingleCookieRemover_1.SingleCookieRemover(cookieName, cookies));
        }
        if (hasEnabledMultipleCookies) {
            return new MultipleCookieRemover_1.MultipleCookieRemover(cookieName, cookies);
        }
        return new SingleCookieRemover_1.SingleCookieRemover(cookieName, cookies);
    }
}
exports.CookieRemoverFactory = CookieRemoverFactory;
