import type { CookieSerializeOptions } from 'cookie';
import type { NextRequest } from 'next/server';
import { NextResponse } from 'next/server';
import { ServiceAccount } from '../auth/credential.js';
import { InvalidTokenReason } from '../auth/error.js';
import { Tokens } from '../auth/index.js';
export interface CreateAuthMiddlewareOptions {
    loginPath: string;
    logoutPath: string;
    cookieName: string;
    cookieSignatureKeys: string[];
    cookieSerializeOptions: CookieSerializeOptions;
    serviceAccount?: ServiceAccount;
    apiKey: string;
    tenantId?: string;
    refreshTokenPath?: string;
    enableMultipleCookies?: boolean;
    enableCustomToken?: boolean;
    authorizationHeaderName?: string;
    experimental_createAnonymousUserIfUserNotFound?: boolean;
    dynamicCustomClaimsKeys?: string[];
}
interface RedirectToPathOptions {
    shouldClearSearchParams: boolean;
}
export declare function redirectToPath(request: NextRequest, path: string, options?: RedirectToPathOptions): NextResponse<unknown>;
interface RedirectToHomeOptions {
    path: string;
}
export declare function redirectToHome(request: NextRequest, options?: RedirectToHomeOptions): NextResponse<unknown>;
export type PublicPath = string | RegExp;
interface RedirectToLoginOptions {
    path: string;
    publicPaths: PublicPath[];
    redirectParamKeyName?: string;
}
export declare function redirectToLogin(request: NextRequest, options?: RedirectToLoginOptions): NextResponse<unknown>;
export declare function createAuthMiddlewareResponse(request: NextRequest, options: CreateAuthMiddlewareOptions): Promise<NextResponse>;
export type HandleInvalidToken = (reason: InvalidTokenReason) => Promise<NextResponse>;
export type HandleValidToken = (tokens: Tokens, headers: Headers) => Promise<NextResponse>;
export type HandleError = (e: unknown) => Promise<NextResponse>;
export interface AuthMiddlewareOptions extends CreateAuthMiddlewareOptions {
    serviceAccount?: ServiceAccount;
    apiKey: string;
    debug?: boolean;
    headers?: Headers;
    checkRevoked?: boolean;
    handleInvalidToken?: HandleInvalidToken;
    handleValidToken?: HandleValidToken;
    handleError?: HandleError;
    experimental_enableTokenRefreshOnExpiredKidHeader?: boolean;
}
export declare function authMiddleware(request: NextRequest, options: AuthMiddlewareOptions): Promise<NextResponse>;
export {};
